// 
// Copyright (C) 2006, NinjaTrader LLC <ninjatrader@ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
	/// <summary>
	/// Elliot Oscillator
	/// </summary>
	[Description("The Elliot Oscillator is a trend following momentum indicator that shows the relationship between two moving averages of prices.")]
	[Gui.Design.DisplayName("Elliot Oscillator")]
	public class ElliotOscillator : Indicator
	{
		#region Variables
		private int					fast	= 5;
		private int					slow	= 34;
		private double 				elliot	= 0;
		private double 				elliot3	= 0;
		private	DataSeries		fastEma;
		private	DataSeries		slowEma;
		#endregion

		/// <summary>
		/// This method is used to configure the indicator and is called once before any bar data is loaded.
		/// </summary>
		protected override void Initialize()
		{
			Add(new Plot(Color.Transparent, "Elliot"));
			Add(new Plot(new Pen(Color.Black, 3), PlotStyle.Bar, "Neutral"));
			Add(new Plot(new Pen(Color.Blue, 3), PlotStyle.Bar, "UpTrend"));
			Add(new Plot(new Pen(Color.Red, 3), PlotStyle.Bar, "DownTrend"));
			Add(new Plot(Color.Red, "Elliot Displaced"));

			Add(new Line(Color.DarkGray, 0, "Zero line"));

			fastEma	= new DataSeries(this);
			slowEma	= new DataSeries(this);
			CalculateOnBarClose	= true;
			PriceTypeSupported	= true;
			AutoScale = true;
		}

		/// <summary>
		/// Calculates the indicator value(s) at the current index.
		/// </summary>
		protected override void OnBarUpdate()
		{
			if (CurrentBar < 3)
				return;
			if (CurrentBar == 0)
			{
				fastEma.Set(Input[0]);
				slowEma.Set(Input[0]);
				Value.Set(0);
				Elliott3.Set(0);
				
				
				if (SMA(100)[0] > SMA(200)[0] && SMA(20)[0] > SMA(100)[0])
					Uptrend.Set(0);
				else if (SMA(100)[0] < SMA(200)[0] && SMA(20)[0] < SMA(100)[0])
					Downtrend.Set(0);
				else
					Neutral.Set(0);
			}
			else
			{
				fastEma.Set(SMA(fast)[0]);
				slowEma.Set(SMA(slow)[0]);
				elliot		= ((fastEma[0] - slowEma[0]));
				elliot3		= ((fastEma[3] - slowEma[3]));
								
				Value.Set(elliot);
				Elliott3.Set(elliot3);
				
				
				if (SMA(100)[0] > SMA(200)[0] && SMA(20)[0] > SMA(100)[0])
					Uptrend.Set(elliot);
				else if (SMA(100)[0] < SMA(200)[0] && SMA(20)[0] < SMA(100)[0])
					Downtrend.Set(elliot);
				else
					Neutral.Set(elliot);
			}
			
		
		}
	
		#region Properties
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Default
		{
			get { return Values[0]; }
		}
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Neutral
		{
			get { return Values[1]; }
		}
        [Browsable(false)]
		[XmlIgnore()]
		public DataSeries Uptrend
		{
			get { return Values[2]; }
		}		
		
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Downtrend
		{
			get { return Values[3]; }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		[XmlIgnore()]
		public DataSeries Elliott3
		{
			get { return Values[4]; }
		}
		
		
	
		
		/// <summary>
		/// </summary>
		[Description("Number of bars for fast SMA")]
		[Category("Parameters")]
		public int Fast
		{
			get { return fast; }
			set { fast = Math.Max(1, value); }
		}

		/// <summary>
		/// </summary>
		[Description("Number of bars for slow SMA")]
		[Category("Parameters")]
		public int Slow
		{
			get { return slow; }
			set { slow = Math.Max(1, value); }
		}
	
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private ElliotOscillator[] cacheElliotOscillator = null;

        private static ElliotOscillator checkElliotOscillator = new ElliotOscillator();

        /// <summary>
        /// The Elliot Oscillator is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public ElliotOscillator ElliotOscillator(int fast, int slow)
        {
            return ElliotOscillator(Input, fast, slow);
        }

        /// <summary>
        /// The Elliot Oscillator is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public ElliotOscillator ElliotOscillator(Data.IDataSeries input, int fast, int slow)
        {
            checkElliotOscillator.Fast = fast;
            fast = checkElliotOscillator.Fast;
            checkElliotOscillator.Slow = slow;
            slow = checkElliotOscillator.Slow;

            if (cacheElliotOscillator != null)
                for (int idx = 0; idx < cacheElliotOscillator.Length; idx++)
                    if (cacheElliotOscillator[idx].Fast == fast && cacheElliotOscillator[idx].Slow == slow && cacheElliotOscillator[idx].EqualsInput(input))
                        return cacheElliotOscillator[idx];

            ElliotOscillator indicator = new ElliotOscillator();
            indicator.SetUp();
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.Fast = fast;
            indicator.Slow = slow;

            ElliotOscillator[] tmp = new ElliotOscillator[cacheElliotOscillator == null ? 1 : cacheElliotOscillator.Length + 1];
            if (cacheElliotOscillator != null)
                cacheElliotOscillator.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheElliotOscillator = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// The Elliot Oscillator is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ElliotOscillator ElliotOscillator(int fast, int slow)
        {
            return _indicator.ElliotOscillator(Input, fast, slow);
        }

        /// <summary>
        /// The Elliot Oscillator is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public Indicator.ElliotOscillator ElliotOscillator(Data.IDataSeries input, int fast, int slow)
        {
            return _indicator.ElliotOscillator(input, fast, slow);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// The Elliot Oscillator is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.ElliotOscillator ElliotOscillator(int fast, int slow)
        {
            return _indicator.ElliotOscillator(Input, fast, slow);
        }

        /// <summary>
        /// The Elliot Oscillator is a trend following momentum indicator that shows the relationship between two moving averages of prices.
        /// </summary>
        /// <returns></returns>
        public Indicator.ElliotOscillator ElliotOscillator(Data.IDataSeries input, int fast, int slow)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.ElliotOscillator(input, fast, slow);
        }

    }
}
#endregion
